@@ -51,6 +51,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 extern int calmaNonManhattan;
 
 extern int CalmaPolygonCount;
+extern int CalmaPathCount;
 extern HashTable calmaDefInitHash;
 
 extern void calmaLayerError();
@@ -443,7 +444,8 @@ calmaElementPath()
     if (nbytes > 0 && rtype == CALMA_PATHTYPE)
 	if (!calmaReadI2Record(CALMA_PATHTYPE, &pathtype)) return;
 
-    if (pathtype != CALMAPATH_SQUAREFLUSH && pathtype != CALMAPATH_SQUAREPLUS)
+    if (pathtype != CALMAPATH_SQUAREFLUSH && pathtype != CALMAPATH_SQUAREPLUS &&
+		pathtype != CALMAPATH_CUSTOM)
     {
 	calmaReadError("Warning: pathtype %d unsupported (ignored).\n", pathtype);
 	pathtype = CALMAPATH_SQUAREFLUSH;
@@ -469,8 +471,15 @@ calmaElementPath()
 
     width /= calmaReadScale2;
 
-    /* Handle BGNEXTN, ENDEXTN */
+    /* Set path extensions based on path type.  Note that SQUARE endcaps */
+    /* are handled automatically by the CIFPaintWirePath routine.	 */
+    /* Round endcaps are not really handled other than assuming they're	 */
+    /* the same as square.  All others are truncated to zero and any	 */
+    /* custom endcap is added to the path here.				 */
+
     extend1 = extend2 = 0;
+
+    /* Handle BGNEXTN, ENDEXTN */
     PEEKRH(nbytes, rtype);
     if (nbytes > 0 && rtype == CALMA_BGNEXTN)
     {
@@ -568,16 +577,16 @@ calmaElementPath()
     {
 	plane = cifCurReadPlanes[ciftype];
 
-	if (CalmaSubcellPolygons && (calmaNonManhattan > 0))
+	if (CalmaSubcellPaths)
 	{
-	    /* Place the polygon in its own subcell */
-	    char newname[] = "polygonXXXXX";
+	    /* Place the path in its own subcell */
+	    char newname[] = "pathXXXXX";
 	    HashEntry *he;
 
 	    savedef = cifReadCellDef;
 
 	    /* Make up name for cell */
-	    sprintf(newname + 7, "%05d", ++CalmaPolygonCount);
+	    sprintf(newname + 4, "%05d", ++CalmaPathCount);
 
 	    he = HashFind(&calmaDefInitHash, newname);
 	    if (!HashGetValue(he))
@@ -597,8 +606,8 @@ calmaElementPath()
 	}
 
 	CIFPaintWirePath(pathheadp, width,
-		(pathtype == CALMAPATH_SQUAREFLUSH) ? FALSE : TRUE,
-		plane, CIFPaintTable, (PaintUndoInfo *)NULL);
+		(pathtype == CALMAPATH_SQUAREFLUSH || pathtype == CALMAPATH_CUSTOM) ?
+		FALSE : TRUE, plane, CIFPaintTable, (PaintUndoInfo *)NULL);
 
 	if (cifCurReadPlanes == cifEditCellPlanes)
 	{
