@@ -1,9 +1,11 @@
 /* grTCairo1.c
  *
- * Copyright (C) 2003 Open Circuit Design, Inc., for MultiGiG Ltd.
+ * Copyright (C) 2017 Open Circuit Design
  *
- * This file contains primitive functions for OpenGL running under
+ * This file contains primitive functions for Cairo running under
  * an X window system in a Tcl/Tk interpreter environment
+ *
+ * Written by Chuan Chen
  */
 
 #include <stdio.h>
@@ -19,10 +21,6 @@
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
 
-/*
-#include <GL/gl.h>
-#include <GL/glx.h>
-*/
 #include <cairo/cairo-xlib.h>
 
 #include "tcltk/tclmagic.h"
@@ -43,21 +41,14 @@
 #include "database/database.h"
 #include "drc/drc.h"
 #include "utils/macros.h"
-//#include "graphics/grTCairoInt.h"
 #include "graphics/grTCairoInt.h"
 #include "utils/paths.h"
 #include "graphics/grTkCommon.h"
 
 uint8_t			**grTCairoStipples;
 HashTable		grTCairoWindowTable;
-//GLXContext	grXcontext;
-cairo_surface_t *grCairoSurface;
-cairo_t 		*grCairoContext;
 XVisualInfo		*grVisualInfo;
 
-//TCairo_CURRENT tcairoCurrent= {(Tk_Font)0, 0, 0, 0, 0,
-//	(Tk_Window)0, (Window)0, (MagWindow *)NULL};
-
 TCAIRO_CURRENT tcairoCurrent = {(Tk_Font)0, 0, 0, 0, 0,
                                 (Tk_Window)0, (Window)0, (MagWindow *)NULL
                                };
@@ -77,9 +68,11 @@ extern void GrTCairoLock(), GrTCairoUnlock(), GrTCairoIconUpdate();
 extern bool GrTCairoInit();
 extern bool GrTCairoEventPending(), GrTCairoCreate(), grtcairoGetCursorPos();
 extern int  GrTCairoWindowId();
-extern char *GrTkWindowName();
+extern char *GrTkWindowName();		/* Use routine from grTkCommon.c */
 
 extern void tcairoSetProjection();
+
+extern int grCurColor;
 
 /*---------------------------------------------------------
  * grtcairoSetWMandC:
@@ -101,52 +94,36 @@ int c;			/* New value for current color */
 {
 	static int oldColor = -1;
 	static int oldMask = -1;
+	TCairoData *tcairodata = (TCairoData *)tcairoCurrent.mw->w_grdata2;
 
 	int lr, lb, lg;
-	//GLfloat fr, fb, fg;
-	//GLfloat aval; 	 /* Alpha default value was 0.75 */
 	float fr, fb, fg, aval;
 
 	if (mask == -65) mask = 127;	/* All planes */
 	if (mask == oldMask && c == oldColor) return;
 
-	//GR_TCairo_FLUSH_BATCH();
 	GR_TCAIRO_FLUSH_BATCH();
 
-	GrGetColor(c, &lr, &lb, &lg);
+	GrGetColor(c, &lr, &lg, &lb);
 
 	fr = ((float)lr / 255);
 	fg = ((float)lg / 255);
 	fb = ((float)lb / 255);
-	aval = ((float)mask / 127.0);
 
-	/*
-		if (mask == 127)
-		{
-			glDisable(GL_BLEND);
-			aval = 1.0;
-		}
-		else
-		{
-			//Calculate a "supercolor", outside of the normal color range,
-			//but which results in the desired color after a blend with
-			//the background color.
+	if (mask == 127)
+	{
+	    aval = 1.0;
+	}
+	else
+	{
+	    /* "Supercolor", to counter the gray background */
+	    fr = fr * 2 - 0.8;
+	    fg = fg * 2 - 0.8;
+	    fb = fb * 2 - 0.8;
+	    aval = ((float)mask / 127.0);
+	}
 
-			fr = fr * 2 - 0.8;
-			fg = fg * 2 - 0.8;
-			fb = fb * 2 - 0.8;
-
-			aval = (GLfloat)mask / 127.0;	// mask translates to alpha in
-			//the OpenGL version.
-
-			glEnable(GL_BLEND);
-			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-		}
-	*/
-
-	//glColor4f(fr, fb, fg, aval);
-
-	cairo_set_source_rgba(grCairoContext, fr, fg, fb, aval);
+	cairo_set_source_rgba(tcairodata->context, fr, fg, fb, aval);
 
 	oldColor = c;
 	oldMask = mask;
@@ -170,28 +147,6 @@ grtcairoSetLineStyle (style)
 int style;			/* New stipple pattern for lines. */
 {
 	// unimplemented for cairo
-	/*
-	static int oldStyle = -1;
-	GLushort glstyle;
-
-	style &= 0xFF;
-	if (style == oldStyle) return;
-	oldStyle = style;
-	GR_TCairo_FLUSH_BATCH();
-
-	switch (style) {
-	case 0xFF:
-	case 0x00:
-		glDisable(GL_LINE_STIPPLE);
-		break;
-	default:
-		glstyle = style | (style << 8);
-		glEnable(GL_LINE_STIPPLE);
-		glLineStipple(1, glstyle);
-		break;
-	}
-	*/
-	;
 }
 
 
@@ -225,7 +180,7 @@ int numstipples;			/* Number of stipples */
 		pdata = (uint8_t *)mallocMagic(128 * sizeof(uint8_t));
 		n = 0;
 
-		/* expand magic's default 8x8 stipple to OpenGL's 32x32 */
+		/* expand magic's default 8x8 stipple to 32x32 */
 
 		for (i = 0; i < 32; i++) {
 			for (j = 0; j < 4; j++) {
@@ -234,8 +189,10 @@ int numstipples;			/* Number of stipples */
 		}
 
 		grTCairoStipples[k] = pdata;
-		stipplePatterns[k] = cairo_pattern_create_for_surface(cairo_image_surface_create_for_data(pdata, CAIRO_FORMAT_A1, 32, 32,
-		                     cairo_format_stride_for_width(CAIRO_FORMAT_A1, 32)));
+		stipplePatterns[k] = cairo_pattern_create_for_surface(
+			cairo_image_surface_create_for_data(pdata,
+			CAIRO_FORMAT_A1, 32, 32,
+			cairo_format_stride_for_width(CAIRO_FORMAT_A1, 32)));
 	}
 }
 
@@ -252,25 +209,29 @@ int numstipples;			/* Number of stipples */
  *---------------------------------------------------------
  */
 
+cairo_pattern_t *currentStipple;
+
 void
 grtcairoSetStipple (stipple)
 int stipple;			/* The stipple number to be used. */
 {
 	static int oldStip = -1;
+	cairo_matrix_t matrix;
+
 	if (stipple == oldStip) return;
 	oldStip = stipple;
 	GR_TCAIRO_FLUSH_BATCH();
 	if (stipple == 0 || stipple > grNumStipples) {
-		//glDisable(GL_POLYGON_STIPPLE);
-		cairo_set_source_rgb(grCairoContext, 0, 0, 0);
+		currentStipple = cairo_pattern_create_rgba(0, 0, 0, 1);
 	} else {
 		if (stipplePatterns[stipple] == (cairo_pattern_t *)NULL) MainExit(1);
-		//glEnable(GL_POLYGON_STIPPLE);
-		//glPolygonStipple(grTCairoStipples[stipple]);
+
+		/* Patterns will be upside-down if not transformed like the window */
+		cairo_matrix_init_scale (&matrix, 1.0, -1.0);
+		cairo_pattern_set_matrix (stipplePatterns[stipple], &matrix);
 		cairo_pattern_set_extend(stipplePatterns[stipple], CAIRO_EXTEND_REPEAT);
 		cairo_pattern_set_filter(stipplePatterns[stipple], CAIRO_FILTER_NEAREST);
-		cairo_set_source(grCairoContext, stipplePatterns[stipple]);
-		cairo_mask_surface(grCairoContext, grCairoSurface, 0.0, 0.0);
+		currentStipple = stipplePatterns[stipple];
 	}
 }
 
@@ -295,15 +256,8 @@ bool
 GrTCairoInit ()
 {
 	bool rstatus;
-	/*
-	#ifdef THREE_D
-		static int attributeList[] = { GLX_RGBA, GLX_DOUBLEBUFFER, None };
-	#else
-		static int attributeList[] = { GLX_RGBA, None, None };
-	#endif
-	*/
 
-	tcairoCurrent.window = Tk_MainWindow(magicinterp); // XDefaultRootWindow(grXdpy) ??
+	tcairoCurrent.window = Tk_MainWindow(magicinterp);
 	if (tcairoCurrent.window == NULL)
 	{
 		TxError("No Top-Level Tk window available. . . is Tk running?\n");
@@ -316,7 +270,6 @@ GrTCairoInit ()
 
 	grXscrn = DefaultScreen(grXdpy);
 
-	//grVisualInfo = glXChooseVisual(grXdpy, grXscrn, attributeList);
 	XVisualInfo grtemplate;
 	int gritems;
 	grtemplate.screen = grXscrn;
@@ -327,42 +280,12 @@ GrTCairoInit ()
 	{
 		TxError("No suitable visual!\n");
 		return FALSE;
-		/*
-		// Try for a double-buffered configuration
-		#ifdef THREE_D
-		attributeList[1] = None;
-		#else
-		attributeList[1] = GLX_DOUBLEBUFFER;
-		#endif
-		grVisualInfo = glXChooseVisual(grXdpy, grXscrn, attributeList);
-		if (!grVisualInfo)
-		{
-		TxError("No suitable visual!\n");
-		return FALSE;
-		}
-		*/
 	}
 
 	grXscrn = grVisualInfo->screen;
 	tcairoCurrent.depth = grVisualInfo->depth;
 
-	/* TRUE = Direct rendering, FALSE = Indirect rendering */
-	/* (note that direct rendering may not be able to deal with pixmaps) */
-	//grXcontext = glXCreateContext(grXdpy, grVisualInfo, NULL, GL_FALSE);
-
-	grCairoSurface = cairo_xlib_surface_create(grXdpy, tcairoCurrent.windowid, grVisualInfo->visual, Tk_Width(tcairoCurrent.window), Tk_Height(tcairoCurrent.window));
-	grCairoContext = cairo_create(grCairoSurface);
-
-	/* Basic GL parameters */
-	/*
-	glLineWidth(1.0);
-	glShadeModel (GL_FLAT);
-	glPixelStorei(GL_PACK_LSB_FIRST, TRUE);
-	*/
-	cairo_set_line_width(grCairoContext, 1.0);
-	cairo_set_source_rgb(grCairoContext, 0, 0, 0);
-
-	/* OpenGL sets its own names for colormap and dstyle file types */
+	/* Use OpenGL names for colormap and dstyle file types */
 	grCMapType = "OpenGL";
 	grDStyleType = "OpenGL";
 
@@ -413,22 +336,15 @@ void
 GrTCairoFlush ()
 {
 	GR_TCAIRO_FLUSH_BATCH();
-	//glFlush();
-	//glFinish();
 }
 
 /*
  *---------------------------------------------------------
  */
 
-//static GLXPixmap glpmap = None;
-Pixmap cairopmap = (Pixmap)NULL;
-
-#define glTransYs(n) (DisplayHeight(grXdpy, grXscrn)-(n))
-
 /*
  *---------------------------------------------------------
- * Set the OpenGL viewport (projection matrix) for a window
+ * Set the Cairo projection matrix for a window
  *---------------------------------------------------------
  */
 
@@ -436,63 +352,35 @@ void
 tcairoSetProjection(llx, lly, width, height)
 int llx, lly, width, height;
 {
-	if (tcairoCurrent.mw->w_flags & WIND_OFFSCREEN)
-	{
-		/*
-		if (glpmap != None) glXDestroyGLXPixmap(grXdpy, glpmap);
-		glpmap = glXCreateGLXPixmap(grXdpy, grVisualInfo,
-				(Pixmap)tcairoCurrent.windowid);
-		glXMakeCurrent(grXdpy, (GLXDrawable)glpmap, grXcontext);
-		*/
-		cairopmap = XCreatePixmap(grXdpy, grXscrn, width, height, tcairoCurrent.depth);
-		grCairoSurface = cairo_xlib_surface_create(grXdpy, cairopmap, grVisualInfo->visual, width, height);
-	}
-	else {
-		//glXMakeCurrent(grXdpy, (GLXDrawable)tcairoCurrent.windowid, grXcontext);
-		grCairoSurface = cairo_xlib_surface_create(grXdpy, tcairoCurrent.windowid, grVisualInfo->visual, Tk_Width(tcairoCurrent.window), Tk_Height(tcairoCurrent.window));
-	}
+    TCairoData *tcairodata = (TCairoData *)tcairoCurrent.mw->w_grdata2;
+    bool offscreen = FALSE;
 
-// #ifndef Cairo_SERVER_SIDE_ONLY
-// 	 For batch-processing lines and rectangles
-// 	glEnableClientState(GL_VERTEX_ARRAY);
-// #endif
+    /* Note that offscreen-drawing comes from the Tk Image	*/
+    /* routines in tkCommon.c and does not have an associated	*/
+    /* surface and context, so we need to make them.		*/
 
-	/* Because this tends to result in thick lines, it has been moved	*/
-	/* the line drawing routine so it can be enabled for individual	*/
-	/* lines.								*/
-	/* glEnable(GL_LINE_SMOOTH); */
+    if (tcairodata == NULL) {
+	tcairodata = (TCairoData *)mallocMagic(sizeof(TCairoData));
+	tcairodata->surface = cairo_xlib_surface_create(grXdpy,
+		tcairoCurrent.windowid, grVisualInfo->visual,
+		width, height);
+	tcairodata->context = cairo_create(tcairodata->surface);
+	tcairodata->backing_context = (ClientData)NULL;
+	tcairodata->backing_surface = (ClientData)NULL;
+	tcairoCurrent.mw->w_grdata2 = (ClientData)tcairodata;
 
-	/* Force draw to front buffer (in case of double-buffered config) */
-	//glDrawBuffer(GL_FRONT);
+	cairo_set_line_width(tcairodata->context, 1.0);
+	/* This should be pulled from STYLE_ERASEALL, not hard-coded */
+	cairo_set_source_rgb(tcairodata->context, 0.8, 0.8, 0.8);
+	currentStipple = cairo_pattern_create_rgba(0, 0, 0, 1);
+	offscreen = TRUE;
+    }
 
-	//glMatrixMode(GL_PROJECTION);
-	//glLoadIdentity();
-	cairo_identify_matrix(grCairoContext);
-
-	//glViewport((GLsizei)llx, (GLsizei)lly, (GLsizei) width, (GLsizei) height);
-	// cairo equivalent??
-
-	/* scale to fit window */
-
-#ifdef CAIRO_INVERT_Y
-	//glScalef(1.0 / (float)(width >> 1), -1.0 / (float)(height >> 1), 1.0);
-	//cairo_scale(grCairoContext, 1.0 / (float)(width >> 1), -1.0 / (float)(height >> 1));
-	cairo_scale(grCairoContext, width, -height);
-#else
-	//glScalef(1.0 / (float)(width >> 1), 1.0 / (float)(height >> 1), 1.0);
-	cairo_scale(grCairoContext, width, height);
-#endif
-
-	/* magic origin maps to window center; move to window origin */
-
-	//glTranslated(-(GLsizei)(width >> 1), -(GLsizei)(height >> 1), 0);
-	cairo_translate(grCairoContext, -(int)(width >> 1), -(int)(height >> 1));
-
-	/* Remaining transformations are done on the modelview matrix */
-
-	//glMatrixMode(GL_MODELVIEW);
-	//glLoadIdentity();
-	cairo_identify_matrix(grCairoContext);
+    cairo_identity_matrix(tcairodata->context);
+    /* Half-pixel translate centers coordinates on pixel */
+    if (offscreen == FALSE) cairo_translate(tcairodata->context, 0.5, -0.5);
+    cairo_translate(tcairodata->context, 0, height);
+    cairo_scale(tcairodata->context, 1.0, -1.0);
 }
 
 
@@ -858,14 +746,24 @@ keys_and_buttons:
 
 		screenRect.r_xbot = ConfigureEvent->x;
 		screenRect.r_xtop = ConfigureEvent->x + width;
-		screenRect.r_ytop = glTransYs(ConfigureEvent->y);
-		screenRect.r_ybot = glTransYs(ConfigureEvent->y + height);
+		screenRect.r_ytop = grTransYs(ConfigureEvent->y);
+		screenRect.r_ybot = grTransYs(ConfigureEvent->y + height);
 
 		need_resize = (screenRect.r_xbot != mw->w_screenArea.r_xbot ||
 		               screenRect.r_xtop != mw->w_screenArea.r_xtop ||
 		               screenRect.r_ybot != mw->w_screenArea.r_ybot ||
 		               screenRect.r_ytop != mw->w_screenArea.r_ytop);
 
+		/* Update Cairo surface */
+
+		if (need_resize)
+		{
+		    TCairoData *tcairodata;
+
+		    tcairodata = (TCairoData *)mw->w_grdata2;
+		    cairo_xlib_surface_set_size(tcairodata->surface, width, height);
+		}
+
 		/* Redraw the window */
 
 		WindReframe(mw, &screenRect, FALSE, FALSE);
@@ -918,7 +816,9 @@ keys_and_buttons:
 		if (mw->w_backingStore != (ClientData)NULL)
 		{
 			Rect surface;
+			(*GrLockPtr)(mw, FALSE);
 			(*GrGetBackingStorePtr)(mw, &screenRect);
+			(*GrUnlockPtr)(mw);
 			WindScreenToSurface(mw, &screenRect, &surface);
 			DBWHLRedrawPrepWindow(mw, &surface);
 			WindDrawBorder(mw, &screenRect);
@@ -941,6 +841,25 @@ keys_and_buttons:
 	}
 }
 
+/* Set for off-screen display */
+
+void
+TCairoOffScreen()
+{
+    GrSetCMapPtr = GrTCairoSetCMap;
+    GrFlushPtr = GrTCairoFlush;
+
+    grDrawLinePtr = grtcairoDrawLine;
+    grSetWMandCPtr = grtcairoSetWMandC;
+    grFillRectPtr = grtcairoFillRect;
+    grSetStipplePtr = grtcairoSetStipple;
+    grSetLineStylePtr = grtcairoSetLineStyle;
+    grFillPolygonPtr = grtcairoFillPolygon;
+
+    if (stipplePatterns == NULL)
+	grtcairoSetSPattern(GrStippleTable, grNumStipples);
+}
+
 
 
 /*---------------------------------------------------------
@@ -1028,20 +947,11 @@ char *mouseFileName;
 	grSetLineStylePtr = grtcairoSetLineStyle;
 	grSetCharSizePtr = grtcairoSetCharSize;
 	grFillPolygonPtr = grtcairoFillPolygon;
-
-#ifdef X11_BACKING_STORE
-	GrFreeBackingStorePtr = grtkFreeBackingStore;
-	GrCreateBackingStorePtr = grtkCreateBackingStore;
-	GrGetBackingStorePtr = grtkGetBackingStore;
-	GrPutBackingStorePtr = grtkPutBackingStore;
-	GrScrollBackingStorePtr = grtkScrollBackingStore;
-#else
 	GrFreeBackingStorePtr = grtcairoFreeBackingStore;
 	GrCreateBackingStorePtr = grtcairoCreateBackingStore;
 	GrGetBackingStorePtr = grtcairoGetBackingStore;
 	GrPutBackingStorePtr = grtcairoPutBackingStore;
 	GrScrollBackingStorePtr = grtcairoScrollBackingStore;
-#endif
 
 	if (execFailed) {
 		TxError("Execution failed!\n");
@@ -1091,7 +1001,7 @@ char *name;
 	char	*windowplace;
 	char	windowname[10];
 	int		x      = w->w_frameArea.r_xbot;
-	int		y      = glTransYs(w->w_frameArea.r_ytop);
+	int		y      = grTransYs(w->w_frameArea.r_ytop);
 	int		width  = w->w_frameArea.r_xtop - w->w_frameArea.r_xbot;
 	int		height = w->w_frameArea.r_ytop - w->w_frameArea.r_ybot;
 	unsigned long        attribmask = CWBackPixel | CWBorderPixel | CWColormap;
@@ -1106,8 +1016,8 @@ char *name;
 		               (unsigned int *)&width, (unsigned int *)&height);
 		w->w_frameArea.r_xbot = x;
 		w->w_frameArea.r_xtop = x + width;
-		w->w_frameArea.r_ytop = glTransYs(y);
-		w->w_frameArea.r_ybot = glTransYs(y + height);
+		w->w_frameArea.r_ytop = grTransYs(y);
+		w->w_frameArea.r_ybot = grTransYs(y + height);
 		WindReframe(w, &(w->w_frameArea), FALSE, FALSE);
 	}
 
@@ -1155,13 +1065,19 @@ char *name;
 	if (tkwind != 0)
 	{
 		bool result;
+		TCairoData *tcairodata;
 
 		GrTCairoFlush();
 
 		tcairoCurrent.window = tkwind;
 		tcairoCurrent.mw = w;
 
+		tcairodata = (TCairoData *)mallocMagic(sizeof(TCairoData));
+		tcairodata->backing_context = NULL;
+		tcairodata->backing_surface = NULL;
+
 		w->w_grdata = (ClientData) tkwind;
+		w->w_grdata2 = (ClientData) tcairodata;
 
 		entry = HashFind(&grTCairoWindowTable, (char *)tkwind);
 		HashSetValue(entry, w);
@@ -1182,8 +1098,16 @@ char *name;
 
 		wind = Tk_WindowId(tkwind);
 		tcairoCurrent.windowid = wind;
-		//glXMakeCurrent(grXdpy, (GLXDrawable)wind, grXcontext);
-		grCairoSurface = cairo_xlib_surface_create(grXdpy, tcairoCurrent.windowid, grVisualInfo->visual, Tk_Width(tcairoCurrent.window), Tk_Height(tcairoCurrent.window));
+		tcairodata->surface = cairo_xlib_surface_create(grXdpy,
+			tcairoCurrent.windowid, grVisualInfo->visual,
+			Tk_Width(tcairoCurrent.window),
+			Tk_Height(tcairoCurrent.window));
+		tcairodata->context = cairo_create(tcairodata->surface);
+
+		cairo_set_line_width(tcairodata->context, 1.0);
+		/* This should be pulled from STYLE_ERASEALL, not hard-coded */
+		cairo_set_source_rgb(tcairodata->context, 0.8, 0.8, 0.8);
+		currentStipple = cairo_pattern_create_rgba(0, 0, 0, 1);
 
 		Tk_DefineCursor(tkwind, tcairoCurrent.cursor);
 		GrTCairoIconUpdate(w, w->w_caption);
@@ -1218,7 +1142,7 @@ char *name;
  * ----------------------------------------------------------------------------
  *
  * GrTCairoDelete --
- *      Destroy a Tk/OpenGL window.
+ *      Destroy a Tk/Cairo window.
  *
  * Results:
  *	None.
@@ -1235,11 +1159,22 @@ MagWindow *w;
 {
 	Tk_Window xw;
 	HashEntry	*entry;
+	TCairoData *tcairodata;
 
 	xw = (Tk_Window) w->w_grdata;
 	entry = HashLookOnly(&grTCairoWindowTable, (char *)xw);
 	HashSetValue(entry, NULL);
 
+	grtcairoFreeBackingStore(w);
+
+	tcairodata = (TCairoData *)w->w_grdata2;
+	if (tcairodata->surface != NULL)
+	    cairo_surface_destroy(tcairodata->surface);
+	if (tcairodata->context != NULL)
+	    cairo_destroy(tcairodata->context);
+	freeMagic(w->w_grdata2);
+	w->w_grdata2 = (ClientData)NULL;
+
 	Tcl_DeleteCommand(magicinterp, Tk_PathName(xw));
 	Tk_DestroyWindow(xw);
 }
@@ -1263,12 +1198,18 @@ void
 GrTCairoConfigure(w)
 MagWindow *w;
 {
+	TCairoData *tcairodata;
+	int width, height;
+
 	if (w->w_flags & WIND_OFFSCREEN) return;
 
+	width = w->w_frameArea.r_xtop - w->w_frameArea.r_xbot;
+	height = w->w_frameArea.r_ytop - w->w_frameArea.r_ybot;
 	Tk_MoveResizeWindow((Tk_Window)w->w_grdata,
-	                    w->w_frameArea.r_xbot, glTransYs(w->w_frameArea.r_ytop),
-	                    w->w_frameArea.r_xtop - w->w_frameArea.r_xbot,
-	                    w->w_frameArea.r_ytop - w->w_frameArea.r_ybot);
+			w->w_frameArea.r_xbot, grTransYs(w->w_frameArea.r_ytop),
+			width, height); 
+	tcairodata = (TCairoData *)w->w_grdata2;
+	cairo_xlib_surface_set_size(tcairodata->surface, width, height);
 }
 
 /*
@@ -1393,8 +1334,7 @@ void
 GrTCairoUnlock(w)
 MagWindow *w;
 {
-	/* GR_TCairo_FLUSH_BATCH(); */
-	GrTCairoFlush();	/* (?) Adds glFlush and glFinish to the above. */
+	GrTCairoFlush();
 	grSimpleUnlock(w);
 }
 
