@@ -299,9 +299,28 @@ GrTOGLInit ()
     grXscrn = grVisualInfo->screen;
     toglCurrent.depth = grVisualInfo->depth;
 
-    /* TRUE = Direct rendering, FALSE = Indirect rendering */
-    /* (note that direct rendering may not be able to deal with pixmaps) */
+    /* glXCreateContext() 4th argument:				*/
+    /* TRUE = Direct rendering, FALSE = Indirect rendering 	*/
+
+    /* Direct rendering may not be able to deal with pixmaps. 	*/
+    /* Normally, magic should compile with OpenGL framebuffer	*/
+    /* backing store and pbuffer offscreen rendering.  However,	*/
+    /* if X11_BACKING_STORE is selected then force indirect	*/
+    /* rendering.  This can be avoided if the backing store	*/
+    /* functions sync between OpenGL and X11;  this has not	*/
+    /* been done yet.						*/
+
+    /* The CAIRO_OFFSCREEN_RENDER compile-time option uses	*/
+    /* Cairo to do the off-screen rendering, which allows	*/
+    /* OpenGL to run in direct-rendering mode only.  To do:	*/
+    /* determine from OpenGL attributes if indirect rendering	*/
+    /* is allowed, and handle automatically.			*/
+
+#ifdef X11_BACKING_STORE  
     grXcontext = glXCreateContext(grXdpy, grVisualInfo, NULL, GL_FALSE);
+#else
+    grXcontext = glXCreateContext(grXdpy, grVisualInfo, NULL, GL_TRUE);
+#endif
 
     /* Basic GL parameters */
 
@@ -362,14 +381,13 @@ GrTOGLFlush ()
 {
     GR_TOGL_FLUSH_BATCH();
     glFlush();
-    glFinish();
 }
 
 /*
  *---------------------------------------------------------
  */
 
-static GLXPixmap glpmap = None;
+static GLXPbuffer pbuffer = None;
 
 #define glTransYs(n) (DisplayHeight(grXdpy, grXscrn)-(n))
 
@@ -385,10 +403,23 @@ toglSetProjection(llx, lly, width, height)
 {
     if (toglCurrent.mw->w_flags & WIND_OFFSCREEN)
     {
-	if (glpmap != None) glXDestroyGLXPixmap(grXdpy, glpmap);
-	glpmap = glXCreateGLXPixmap(grXdpy, grVisualInfo,
-			(Pixmap)toglCurrent.windowid);
-	glXMakeCurrent(grXdpy, (GLXDrawable)glpmap, grXcontext);
+	int count = 0;
+	int PBattrib[] = {
+	    GLX_PBUFFER_WIDTH, width,
+	    GLX_PBUFFER_HEIGHT, height,
+	    GLX_LARGEST_PBUFFER, False,
+	    None
+	};
+	GLXFBConfig *config;
+
+	if (pbuffer != None) glXDestroyPbuffer(grXdpy, pbuffer);
+	config = glXGetFBConfigs(grXdpy, grXscrn, &count);
+	if (config != NULL && count != 0)
+	{
+	    pbuffer = glXCreatePbuffer(grXdpy, config[0], PBattrib);
+	    glXMakeCurrent(grXdpy, (GLXDrawable)pbuffer, grXcontext);
+	}
+	if (config != NULL) XFree(config);
     }
     else
 	glXMakeCurrent(grXdpy, (GLXDrawable)toglCurrent.windowid, grXcontext);
@@ -423,6 +454,9 @@ toglSetProjection(llx, lly, width, height)
 
     glTranslated(-(GLsizei)(width >> 1), -(GLsizei)(height >> 1), 0);
 
+    if (toglCurrent.mw->w_flags & WIND_OFFSCREEN)
+	glTranslatef((float)0.5, (float)0.5, 0);
+
     /* Remaining transformations are done on the modelview matrix */
 
     glMatrixMode(GL_MODELVIEW);
@@ -852,7 +886,9 @@ keys_and_buttons:
 		if (mw->w_backingStore != (ClientData)NULL)
 		{
 		    Rect surface;
+		    (*GrLockPtr)(mw, FALSE);
 		    (*GrGetBackingStorePtr)(mw, &screenRect);
+		    (*GrUnlockPtr)(mw);
 		    WindScreenToSurface(mw, &screenRect, &surface);
 		    DBWHLRedrawPrepWindow(mw, &surface);
 		    WindDrawBorder(mw, &screenRect);
@@ -877,6 +913,23 @@ keys_and_buttons:
 
 
 
+/* Set for on-screen display */
+
+void
+toglOnScreen()
+{
+    GrSetCMapPtr = GrTOGLSetCMap;
+    GrFlushPtr = GrTOGLFlush;
+
+    grSetSPatternPtr = grtoglSetSPattern;
+    grDrawLinePtr = grtoglDrawLine;
+    grSetWMandCPtr = grtoglSetWMandC;
+    grFillRectPtr = grtoglFillRect;
+    grSetStipplePtr = grtoglSetStipple;
+    grSetLineStylePtr = grtoglSetLineStyle;
+    grFillPolygonPtr = grtoglFillPolygon;
+}
+
 /*---------------------------------------------------------
  * oglSetDisplay:
  *	This routine sets the appropriate parameters so that
@@ -1277,6 +1330,11 @@ GrTOGLLower(w)
  * ----------------------------------------------------------------------------
  */
 
+#ifdef CAIRO_OFFSCREEN_RENDER
+extern void GrTCairoLock();
+extern void TCairoOffScreen();
+#endif
+
 void
 GrTOGLLock(w, flag)
     MagWindow *w;
@@ -1284,6 +1342,17 @@ GrTOGLLock(w, flag)
 {
     Window wind;
 
+#ifdef CAIRO_OFFSCREEN_RENDER
+    /* Use Cairo graphics for off-screen rendering */
+
+    if ((w != GR_LOCK_SCREEN) && (w->w_flags & WIND_OFFSCREEN))
+    {
+	GrTCairoLock(w, flag);
+	TCairoOffScreen();
+	return;
+    }
+#endif
+
     grSimpleLock(w, flag);
     if ( w != GR_LOCK_SCREEN )
     {
@@ -1326,12 +1395,74 @@ void
 GrTOGLUnlock(w)
     MagWindow *w;
 {
-    /* GR_TOGL_FLUSH_BATCH(); */
-    GrTOGLFlush();	/* (?) Adds glFlush and glFinish to the above. */
+
+#ifdef CAIRO_OFFSCREEN_RENDER
+    /* Use Cairo graphics for off-screen rendering */
+
+    if ((w != GR_LOCK_SCREEN) && (w->w_flags & WIND_OFFSCREEN))
+    {
+	GrTCairoUnlock(w);
+	toglOnScreen();
+	return;
+    }
+#endif
+
+    GrTOGLFlush();
+
+    if ((w != GR_LOCK_SCREEN) && (w->w_flags & WIND_OFFSCREEN))
+    {
+	GC grXcopyGC;
+	XGCValues gcValues;
+	unsigned char *pdata, *tdata;
+	int i, j;
+
+        Window root_return;
+        int x_return, y_return;
+        unsigned int pbwidth, pbheight, wborder, depth;
+
+	XGetGeometry(grXdpy, (Drawable)toglCurrent.windowid, &root_return,
+		&x_return, &y_return, &pbwidth, &pbheight, &wborder, &depth);
+
+	pdata = (unsigned char *)mallocMagic((pbwidth * pbheight * 3)
+			* sizeof(unsigned int));
+
+	/* In offscreen-rendering mode, copy Pbuffer back to window */
+	glReadBuffer(GL_FRONT);
+	glReadPixels(0, 0, pbwidth, pbheight, GL_RGB, GL_UNSIGNED_BYTE, pdata);
+
+        gcValues.graphics_exposures = FALSE;
+        grXcopyGC = XCreateGC(grXdpy, (Drawable)toglCurrent.windowid,
+		GCGraphicsExposures, &gcValues);
+
+	/* This is very slow, but the only way I've found to copy data	*/
+	/* from a Pbuffer into a Pixmap.  It is only used to make the	*/
+	/* icon images for the toolbar, so it does not need to be	*/
+	/* efficient.							*/
+
+	tdata = pdata;
+	for (i = 0; i < pbwidth; i++) {
+	    for (j = 0; j < pbheight; j++)
+	    {
+		unsigned long pcolor;
+		pcolor = *tdata++;
+		pcolor <<= 8;
+		pcolor |= *tdata++;
+		pcolor <<= 8;
+		pcolor |= *tdata++;
+		XSetForeground(grXdpy, grXcopyGC, pcolor);
+		XDrawPoint(grXdpy, (Drawable)toglCurrent.windowid, grXcopyGC,
+			pbwidth - i - 1, j);
+	    }
+	}
+
+	freeMagic(pdata);
+	XFreeGC(grXdpy, grXcopyGC);
+    }
     grSimpleUnlock(w);
 }
 
 
+
 /*           
  *-------------------------------------------------------------------------
  * GrTOGLEventPending --
