@@ -315,10 +315,17 @@ lefEraseGeometry(tile, cdata)
     lefClient *lefdata = (lefClient *)cdata;
     CellDef *flatDef = lefdata->lefFlat;
     Rect area;
-    TileType ttype;
+    TileType ttype, otype;
 
     TiToRect(tile, &area);
 
+    otype = TiGetTypeExact(tile);
+    if (IsSplit(tile))
+	ttype = (otype & TT_SIDE) ? SplitRightType(tile) :
+			SplitLeftType(tile);
+    else
+	ttype = otype;
+
     /* Erase the tile area out of lefFlat */
     DBErase(flatDef, &area, ttype);
 
@@ -747,7 +754,7 @@ lefWriteMacro(def, f, scale)
 
     if (!TTMaskIsZero(&boundmask))
     {
-	for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
+	for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
 	    DBSrPaintArea((Tile *)NULL, def->cd_planes[pNum], 
 			&TiPlaneRect, &boundmask, lefGetBound,
 			(ClientData)(&boundary));
@@ -915,7 +922,7 @@ lefWriteMacro(def, f, scale)
 
 	lc.numWrites = 0;
 	lc.lastType = TT_SPACE;
-	for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
+	for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
 	{
 	    lc.pNum = pNum;
 	    DBSrPaintArea((Tile *)NULL, SelectDef->cd_planes[pNum], 
@@ -970,8 +977,9 @@ lefWriteMacro(def, f, scale)
 
     /* Restrict to routing planes only */
 
-    for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
+    for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
     {
+	lc.pNum = pNum;
 	DBSrPaintArea((Tile *)NULL, lefFlatDef->cd_planes[pNum], 
 		&TiPlaneRect, &DBAllButSpaceAndDRCBits,
 		lefYankGeometry, (ClientData) &lc);
